<?php declare(strict_types=1);

namespace Everyware\Plugin\CacheInvalidator;

/**
 * Class Startup
 * @package Everyware\Plugin\CacheInvalidator
 */
class Startup
{
    public function __construct()
    {
        add_action('admin_notices', function () {
            $this->checkPluginDependencies();
        });

        // Register pages for menu.
        add_action('admin_menu', function () {

            $pageDebug = new PageDebug();
            $pageSettings = new PageSettings();
            $pageEventLog = new PageEventLog();

            add_menu_page(__('EW cache invalidator', 'ew-cache-invalidator'), 'EW Cache', 'manage_options',
                'ew-cache-invalidator', '', 'dashicons-admin-settings');
            add_submenu_page('ew-cache-invalidator', __('Cache invalidation settings', 'ew-cache-invalidator'),
                __('Settings', 'ew-cache-invalidator'), 'manage_options', 'ew-cache-invalidator',
                [$pageSettings, 'renderAdminPage']);
            add_submenu_page('ew-cache-invalidator', __('Cache invalidation data', 'ew-cache-invalidator'),
                __('Debug data', 'ew-cache-invalidator'), 'manage_options', 'ew-cache-invalidator-debug',
                [$pageDebug, 'renderAdminPage']);
            add_submenu_page('ew-cache-invalidator', __('Cache event log', 'ew-cache-invalidator'),
                __('Event log', 'ew-cache-invalidator'), 'manage_options', 'ew-cache-invalidator-event-log',
                [$pageEventLog, 'renderAdminPage']);
        });

        $this->setupAjax();
        if (PageSettings::checkQueryEventsActive()) {
            new EventHandler();
        }
    }

    private function checkPluginDependencies(): void
    {
        $messages = [];

        if ( ! defined('EVERY_VERSION')) {
            $messages[] = '<strong>Everyware Cache Invalidator</strong> — This plugin needs Everyware to be active in order to work properly.';
        }

        if ( ! defined('EVERYBOARD_VERSION')) {
            $messages[] = '<strong>Everyware Cache Invalidator</strong> — This plugin needs Everyboard to be active in order to work properly.';
        }

        foreach ($messages as $message) {
            print '<div class="notice notice-warning is-dismissible"><p>' . $message . '</p></div>';
        }
    }

    /**
     * Setup ajax functions.
     */
    private function setupAjax(): void
    {
        add_action('wp_ajax_ew_cache_invalidator_save_query_streamer', function () {
            PageSettings::ajaxSaveQueryStream();
        });

        add_action('wp_ajax_ew_cache_invalidator_save_query_suffix', function () {
            PageSettings::ajaxSaveQuerySuffix();
        });

        add_action('wp_ajax_ew_cache_invalidator_reset_all_queries', function () {
            PageSettings::ajaxResetAllQueries();
        });

        add_action('wp_ajax_ew_cache_invalidator_delete_all_queries', function () {
            PageSettings::ajaxDeleteAllQueries();
        });

        add_action('wp_ajax_ew_cache_invalidator_check_endpoint_status', function () {
            PageSettings::ajaxCheckEndpointStatus();
        });

        add_action('wp_ajax_ew_cache_invalidator_check_query_streamer_status', function () {
            PageSettings::ajaxCheckQueryStreamStatus();
        });

        add_action('wp_ajax_ew_cache_invalidator_save_event_handling', function () {
            PageSettings::ajaxSaveEventHandling();
        });
    }
}
